function [goodComputation,delta]=lorenzApostori(N,L,sigma,rho,beta,domRad, ...
    P1,P2,P3,lambda_1,lambda_2);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Compute and Validate Parameterization in Gray-Scott
%
%
%-------------NOTE:--------------------------------------------------------
%The main `knobs' that can be adjusted in the computation are:
%N               -> The parameterization order
%domRad          -> The parameterization domain we try to 
%                   validate the estimates on
%L               -> How long the connecting orbit will be.
%
%If the validation fails, then you can either increase the order, decrease
%scaleEigs, or decrease domRad.  The only constraint is that N > 0, domRad
%needs to be greater than one (if we want exponentially decreasing bound)
%on the coefficients).  The smaller scaleEigs is, the smaller the image
%of the parameterizaiotn function will be, i.e. the smaller the local
%stable manifold will be, and the bigger L has to be. If you want to 
%decrease L, you can either increase N, increase domRad, or increase 
%scaleEigs.  However, any of these can destroy the estimates.  So there is
%still something of an art to choosing the right constants.  However,
%once you set them, the program will automatically tell you if they 
%are acceptable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Ws Parameterization Control:

% s determines the distance between rho and rho'
s = 0.0001;

%==========================================================================
%A-Posteriori Validation of parameterization of stable manifold:
%==========================================================================

%==========================================================================
%A-Posteriori Calculation:
%
%Set up for validation of stable manifold of p = [1 0 0 0] in Gray-Scott.


%Code will have to be modified (slightly) to do unstable manifold or
%to do manifolds at other fixed points.
%==========================================================================

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Composition of the vector field with the Parameterization (as powerseries)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%The Hex vector field is 
%               | P_2                                                           |
%               | L^2 [-gamma/4 P_1 + sqrt(2)/4 P_3^2 + 3/8 P_1^3 3 P_1 P_3^2]  |
% g circ P   =  |                                                               |
%               |P_4                                                            |
%               |(^2[-gamma P_3 + sqrt(2)/2 P_1 P_3 + 9 P_3^3 + 3 P_1^2 P_3 ]   |
%


%Shift fixed point to origin:

%The nonlinear terms are evaluated using convolution sums


Pxy = convolveSeries_2D_general(P1,P2);
Pxz = convolveSeries_2D_general(P1,P3);


P1_big = zeros(2*N+1, 2*N+1);
P2_big = zeros(2*N+1, 2*N+1);
P3_big = zeros(2*N+1, 2*N+1);

P1_big(1:(N+1), 1:(N+1)) = P1;
P2_big(1:(N+1), 1:(N+1)) = P2;
P3_big(1:(N+1), 1:(N+1)) = P3;

%Then:
g_of_P1 = L*sigma*(P2_big - P1_big);
g_of_P2 = L*(rho*P1_big - P2_big - Pxz);
g_of_P3 = L*(Pxy - beta*P3_big);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Composition of DP with L(theta):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DP_circL_theta_1 = zeros(2*N+1, 2*N+1);
DP_circL_theta_2 = zeros(2*N+1, 2*N+1);
DP_circL_theta_3 = zeros(2*N+1, 2*N+1);


for n = 0:N
   for m = 0:N
      if ((n == 0) && (m == 0))
        DP_circL_theta_1(1,1) = 0;
        DP_circL_theta_2(1,1) = 0;
        DP_circL_theta_3(1,1) = 0;
      elseif ((n == 0) && (m == 1))
        DP_circL_theta_1(2,1) = lambda_1*P1(2,1);
        DP_circL_theta_2(2,1) = lambda_1*P2(2,1);
        DP_circL_theta_3(2,1) = lambda_1*P3(2,1);
      elseif ((n == 1) && (m == 0)) 
        DP_circL_theta_1(1,2) = lambda_2*P1(1,2);
        DP_circL_theta_2(1,2) = lambda_2*P2(1,2);
        DP_circL_theta_3(1,2) = lambda_2*P3(1,2);
      else
        DP_circL_theta_1(m+1,n+1) = (m*lambda_1+n*lambda_2)*P1(m+1,n+1);
        DP_circL_theta_2(m+1,n+1) = (m*lambda_1+n*lambda_2)*P2(m+1,n+1);
        DP_circL_theta_3(m+1,n+1) = (m*lambda_1+n*lambda_2)*P3(m+1,n+1);
      end
   end
end

%Computation of difference = power series of error function:
Error1 = g_of_P1 - DP_circL_theta_1;
Error2 = g_of_P2 - DP_circL_theta_2;
Error3 = g_of_P3 - DP_circL_theta_3;

%Compute the weighted sum-norm for the error functional:
% Item 1 of Definition 4.1 (Validation Values)
error_weightedNorm = 0;

for m = 0:(2*N)
    for n = 0:(2*N)
       if (m+n > 1) 
          norm_a_mn = norm([Error1(m+1,n+1),Error2(m+1,n+1),Error3(m+1,n+1)]);
          thisTerm = norm_a_mn*(domRad)^(m+n);
          error_weightedNorm = error_weightedNorm + thisTerm;
       end
    end
end



%Consider also the weighted norm of P_N itself, with the same weight,
%centered around the fixed point (hence the first term is zero):
% Item 2 of Definition 4.1 (Validation Values)

P_weightedNorm = 0;
for m = 0:N
    for n = 0:N
        if ((n == 0) && (m == 0))
          P_weightedNorm = 0;  
        else
          norm_a_mn=norm([P1(m+1,n+1),P2(m+1,n+1),P3(m+1,n+1)]);
          thisTerm = norm_a_mn*(domRad)^(m+n);
          P_weightedNorm = P_weightedNorm + thisTerm;
        end
    end
end

epsilon_tol = error_weightedNorm
P_supNorm_bound = P_weightedNorm

rho_prime = P_supNorm_bound
rho = rho_prime + s


goodComputation = 1;
delta = epsilon_tol;